import { logWarn, yellow, registerPlugin } from '@scullyio/scully';
import highlight from 'highlight.js';
import { __awaiter } from 'tslib';
import { readFileSync } from 'fs';
import { JSDOM } from 'jsdom';
import { parse } from 'marked';

const scullyBegin = '<!--scullyContent-begin-->';
const scullyEnd = '<!--scullyContent-end-->';
function renderPlugin(html, route) {
    return __awaiter(this, void 0, void 0, function* () {
        const sectionBuilders = route.config.sectionBuilders;
        try {
            let attr = '';
            try {
                attr = getIdAttrName(html.split('<scully-content')[1].split('>')[0].trim());
            }
            catch (e) {
                logWarn(`
        ----------------
        Error, missing "${yellow('<scully-content>')}" in route "${yellow(route.route)}"
        without <scully-content> we can not render this route.
        Make sure it is in there, and not inside any conditionals (*ngIf)
        You can check this by opening "${yellow(`://localhost:4200/${route.route}`)}"
        when you serve your app with ${yellow('ng serve')} and then in the browsers console run:
        ${yellow(`document.querySelector('scully-content')`)}
        ----------------
        `);
            }
            let additionalHTML = '';
            for (const builder of sectionBuilders) {
                const markdownHtmls = [];
                for (const markdownFileSource of builder.markdownFileSources) {
                    const originalFile = readFileSync(markdownFileSource, 'utf-8');
                    const options = {
                        highlight: (code, language) => {
                            const hljs = highlight;
                            const validLanguage = hljs.getLanguage(language)
                                ? language
                                : 'plaintext';
                            return hljs.highlight(validLanguage, code).value;
                        },
                        pedantic: false,
                        gfm: true,
                        breaks: false,
                        sanitize: false,
                        smartLists: true,
                        smartypants: false,
                        xhtml: false
                    };
                    let markdownHtml = parse(originalFile, options);
                    markdownHtmls.push(markdownHtml);
                }
                additionalHTML = additionalHTML + builder.sectionBuilder(markdownHtmls);
            }
            const htmlWithNgAttr = addNgIdAttribute(additionalHTML, attr);
            return insertContent(scullyBegin, scullyEnd, html, htmlWithNgAttr, getScript(attr));
        }
        catch (e) {
            logWarn(`Error, while rendering content for "${yellow(route.route)}" from files: "${yellow(sectionBuilders.map(builder => builder.markdownFileSources))}"`);
            console.error(e);
        }
    });
}
function addNgIdAttribute(html, id) {
    if (!id) {
        return html;
    }
    try {
        const dom = new JSDOM(html, { runScripts: 'outside-only' });
        const { window } = dom;
        const { document } = window;
        const walk = document.createTreeWalker(document.body);
        let cur = walk.currentNode;
        while (cur) {
            if (cur.nodeType === 1) {
                cur.setAttribute(id, '');
            }
            cur = walk.nextNode();
        }
        return document.body.innerHTML;
    }
    catch (e) {
        console.error(e);
    }
    return '';
}
function getIdAttrName(attrs) {
    try {
        return (attrs &&
            attrs
                .split(' ')
                .find((at) => at.trim().startsWith('_ngcontent'))
                .split('=')[0]);
    }
    catch (_a) {
        return '6';
    }
}
function getScript(attr) {
    // tslint:disable-next-line:no-unused-expression
    return `<script>try {window['scullyContent'] = {cssId:"${attr}",html:document.body.innerHTML.split('<!--scullyContent-begin-->')[1].split('<!--scullyContent-end-->')[0]};} catch(e) {console.error('scully could not parse content');}</script>`;
}
function insertContent(startTag, endTag, html, insertText, ...extras) {
    try {
        const [openingText, rest] = html.split(startTag);
        const [takeout, endText] = rest.split(endTag);
        return [openingText, startTag, insertText, endTag, ...extras, endText].join('');
    }
    catch (e) { }
    /** warning is already handled, only put in stub content. */
    // logWarn(`missing "${yellow('<scully-content>')}"`);
    return `<h1>Scully could not find the &lt.scully-content&gt. tag in this page.</h1>
  <p>This error can happen when you forgot to put the  mandatory "scully-content" in the component that is rendering this page?</p>
  <p>Or when the tag is not shown on page load. Did you put it inside an \`*ngIf\`?</p>
  `;
}

const MultiMarkdownPageBuilderPluginName = 'MultiMarkdownPageBuilderPlugin';

function routerPlugin(route, config) {
    const additionalPostRenderers = config.postRenderers ? config.postRenderers : [];
    const routes = [
        {
            type: MultiMarkdownPageBuilderPluginName,
            route: route === '' ? '/' : route,
            config,
            postRenderers: [MultiMarkdownPageBuilderPluginName, ...additionalPostRenderers],
            title: config.title
        }
    ];
    return Promise.resolve(routes);
}

const renderValidator = (config) => __awaiter(void 0, void 0, void 0, function* () { return []; });

const routerValidator = (config) => __awaiter(void 0, void 0, void 0, function* () {
    if (config.sectionBuilders === undefined || !Array.isArray(config.sectionBuilders)) {
        return [`Missing a sectionBuilders array in the ${MultiMarkdownPageBuilderPluginName} config.`];
    }
    else {
        return config.sectionBuilders.reduce((errors, sectionBuilder) => {
            if (sectionBuilder.sectionBuilder === undefined || !(sectionBuilder.sectionBuilder instanceof Function)) {
                errors.push(`Missing sectionBuilder function type in section builder: ${sectionBuilder}`);
            }
            if (sectionBuilder.markdownFileSources === undefined || !Array.isArray(sectionBuilder.markdownFileSources)) {
                errors.push(`Missing markdownFileSources array type in section builder: ${sectionBuilder}`);
            }
            return errors;
        }, []);
    }
});

const MultiMarkdownPageBuilderPlugin = MultiMarkdownPageBuilderPluginName;
registerPlugin('render', MultiMarkdownPageBuilderPlugin, renderPlugin, renderValidator);
registerPlugin('router', MultiMarkdownPageBuilderPlugin, routerPlugin, routerValidator);

/**
 * Creates a section of a page from a single markdown file, and adds the passed css selectors for inserting layout customization.
 *
 * Example markdown source file:
 * ```md
 * # Markdown Title Content
 * ```
 * Example usage:
 * ```typescript
 * export const config: ScullyConfig = {
 *   // ...
 *   routes: {
 *     '/path': {
 *       type: MultiMarkdownPageBuilderPlugin,
 *       sectionBuilders: [
 *         createSingleMarkdownSection('./homepage/installation.md', { containerDivId: 'installationContainer', elementDivId: 'installationItem' }),
 *       ]
 *     }
 *   }
 * };
 * ```
 * Example output:
 * ```html
 * <div id="installationContainer">
 *  <div id="installationItem">
 *      <h1>Markdown Title Content</h1>
 *  </div>
 * </div>
 * ```
 * @param {string} markdownFileSource source of the markdown file to be converted into HTML, then passed into the sectionBuilder
 * @param {SingleMarkdownSectionBuilderExtras} extras The `containerIdAttribute` will be attached as the `id` attribute value for the container div.  The `elementIdAttribute` will be attached as the `id` attribute value for the interior element div.  Passing `null` for either will omit the selector.
 * @returns A `MarkdownSectionBuilder` that will be used by the plugin to create the single markdown section.
 */
function createSingleMarkdownSection(markdownFileSource, extras) {
    return {
        sectionBuilder: (markdownHtmls) => {
            const containerIdAttribute = extras.containerDivId === null ? '' : ` id="${extras.containerDivId}"`;
            const elementIdAttribute = extras.elementDivId === null ? '' : ` id="${extras.elementDivId}"`;
            return `<div${containerIdAttribute}><div${elementIdAttribute}>${markdownHtmls[0]}</div></div>`;
        },
        markdownFileSources: [markdownFileSource],
        extras
    };
}

/**
 * Creates a section of a page from multiple markdown files, and adds the passed css selectors for inserting layout customization.
 *
 * Example markdown source files:
 * ```md
 * // ./sourceDir/file1.md
 * # First Markdown Title Content
 * ```
 * ```md
 * // ./sourceDir/file2.md
 * # Second Markdown Title Content
 * ```
 * Example usage:
 * ```typescript
 * export const config: ScullyConfig = {
 *   // ...
 *   routes: {
 *     '/path': {
 *       type: MultiMarkdownPageBuilderPlugin,
 *       sectionBuilders: [
 *         createMultipleMarkdownSection(['./sourceDir/file1.md', './sourceDir/file2.md'], { containerDivId: 'gridContainer', elementsDivClass: 'grid-item' })
 *       ]
 *     }
 *   }
 * };
 * ```
 * Example output:
 * ```html
 * <div id="gridContainer">
 *  <div id="element1" class="grid-item">
 *      <h1>First Markdown Title Content</h1>
 *  </div>
 *  <div id="element2" class="grid-item">
 *      <h1>Second Markdown Title Content</h1>
 *  </div>
 * </div>
 * ```
 * @param {string[]} markdownFileSources source of the markdown files to be converted into HTML, then passed into the sectionBuilder
 * @param {SingleMarkdownSectionBuilderExtras} extras The `containerIdAttribute` will be attached as the `id` attribute value for the container div.  The `elementsDivClass` will be attached as the `class` attribute value for the interior element div.  Passing `null` for either will omit the selector.
 * @returns A `MarkdownSectionBuilder` that will be used by the plugin to create the single markdown section.
 */
function createMultipleMarkdownSection(markdownFileSources, extras) {
    return {
        sectionBuilder: (markdownHtmls) => {
            let markdownFileIndex = 1;
            const combinedMarkdownHtml = markdownHtmls.reduce((combined, currentMarkdownHtml) => {
                const idAttribute = ` id="element${markdownFileIndex++}"`;
                const classAttribute = extras.elementsDivClass === null ? '' : ` class="${extras.elementsDivClass}"`;
                return combined + `<div${idAttribute}${classAttribute}>${currentMarkdownHtml}</div>`;
            }, '');
            const idAttribute = extras.containerDivId === null ? '' : ` id="${extras.containerDivId}"`;
            return `<div${idAttribute}>${combinedMarkdownHtml}</div>`;
        },
        markdownFileSources: markdownFileSources,
        extras
    };
}

/*
 * Public API Surface of scully-multi-markdown-page-builder-plugin
 */

/**
 * Generated bundle index. Do not edit.
 */

export { MultiMarkdownPageBuilderPlugin, createMultipleMarkdownSection, createSingleMarkdownSection };
//# sourceMappingURL=scully-multi-markdown-page-builder-plugin.js.map
